@@ -47,6 +47,7 @@ struct database_criterion_value {
 };
 
 struct database_criterion {
+    int union_filter;
     char *name;
     enum pazpar2_database_criterion_type type;
     struct database_criterion_value *values;
@@ -141,13 +142,14 @@ int match_zurl(const char *zurl, const char *pattern)
 
 // This will be generalized at some point
 static int match_criterion(struct setting **settings,
+                           int num_settings,
                            struct conf_service *service,
                            struct database_criterion *c)
 {
     int offset = settings_lookup_offset(service, c->name);
     struct database_criterion_value *v;
 
-    if (offset < 0)
+    if (offset < 0 || offset >= num_settings)
     {
         yaz_log(YLOG_WARN, "Criterion not found: %s", c->name);
         return 0;
@@ -189,9 +191,15 @@ static struct database_criterion *create_database_criterion(NMEM m,
     char **values;
     int num;
     int i;
+    int union_filter = 0;
 
     if (!buf || !*buf)
         return 0;
+    if (*buf == '|')
+    {
+        union_filter = 1;
+        buf++;
+    }
     nmem_strsplit(m, ",", buf,  &values, &num);
     for (i = 0; i < num; i++)
     {
@@ -200,6 +208,8 @@ static struct database_criterion *create_database_criterion(NMEM m,
         int subi;
         struct database_criterion *new = nmem_malloc(m, sizeof(*new));
         char *eq;
+
+        new->union_filter = union_filter;
         for (eq = values[i]; *eq; eq++)
             if (*eq == '=')
             {
@@ -235,16 +245,24 @@ static struct database_criterion *create_database_criterion(NMEM m,
 }
 
 static int database_match_criteria(struct setting **settings,
+                                   int num_settings,
                                    struct conf_service *service,
                                    struct database_criterion *cl)
 {
-    for (; cl; cl = cl->next)
-        if (!match_criterion(settings, service, cl))
-            break;
-    if (cl) // one of the criteria failed to match -- skip this db
+    if (cl && cl->union_filter)
+    {
+        for (; cl; cl = cl->next)
+            if (match_criterion(settings, num_settings, service, cl))
+                return 1;
         return 0;
+    }
     else
-        return 1;
+    {
+        for (; cl; cl = cl->next)
+            if (!match_criterion(settings, num_settings, service, cl))
+                return 0;
+    }
+    return 1;
 }
 
 // Cycles through databases, calling a handler function on the ones for
@@ -263,7 +281,8 @@ int session_grep_databases(struct session *se, const char *filter,
             continue;
         if (!p->settings[PZ_NAME])
             continue;
-        if (database_match_criteria(p->settings, se->service, cl))
+        if (database_match_criteria(p->settings, p->num_settings,
+                                    se->service, cl))
         {
             (*fun)(se, p);
             i++;
@@ -280,7 +299,7 @@ int predef_grep_databases(void *context, struct conf_service *service,
     int i = 0;
 
     for (p = service->databases; p; p = p->next)
-        if (database_match_criteria(p->settings, service, 0))
+        if (database_match_criteria(p->settings, p->num_settings, service, 0))
         {
             (*fun)(context, p);
             i++;
